use crate::list_overview::models::{Current, ListOverviewRequest, Origin};
use crate::{Print, Settings};

pub fn init<'a>(settings: &'a Settings, console: &'a dyn Print) -> OverviewHandler<'a> {
    let overview_gateway = OverviewGateway::new(settings);
    OverviewHandler {
        settings,
        overview_gateway,
        console,
    }
}

pub struct OverviewHandler<'a> {
    pub settings: &'a Settings,
    console: &'a dyn Print,
    overview_gateway: OverviewGateway<'a>,
}

pub struct OverviewGateway<'a> {
    pub settings: &'a Settings,
}

impl<'a> OverviewHandler<'a> {
    pub fn execute(&self, request: ListOverviewRequest) {
        let currents = self.overview_gateway.get_overview(request.count);

        println!("All currents:");
        for current in currents {
            self.console.println(current.name.as_str());
        }
    }
}

impl<'a> OverviewGateway<'a> {
    pub fn new(settings: &Settings) -> OverviewGateway {
        OverviewGateway { settings }
    }

    pub fn get_overview(&self, count: usize) -> Vec<Current> {
        let connection = self.settings.database.get_connection();

        let mut prepared = connection.prepare("SELECT name FROM Currents").unwrap();
        let iter = prepared
            .query_map([], |row| {
                Ok(Current {
                    name: row.get(0).unwrap(),
                    origin: Origin {},
                })
            })
            .unwrap();

        iter.map(|x| x.unwrap()).collect()
    }
}

pub mod models {
    pub struct ListOverviewRequest {
        pub count: usize,
    }

    pub struct Current {
        pub name: String,
        pub origin: Origin,
    }

    pub struct Origin {}

    pub enum VersionCheck {
        Match {
            version: String,
            current: Current,
        },
        Diverging {
            version: String,
            most_recent: Origin,
        },
    }
}
