use crate::configuration::VcsType;
use crate::register_current::models::CurrentRequest;
use crate::Settings;

pub fn init(settings: &Settings) -> RegisterCurrentHandler {
    let current_gateway = CurrentGateway::new(settings);
    RegisterCurrentHandler {
        settings,
        current_gateway,
    }
}

pub struct RegisterCurrentHandler<'a> {
    pub settings: &'a Settings,
    pub current_gateway: CurrentGateway<'a>,
}

pub struct CurrentGateway<'a> {
    pub settings: &'a Settings,
}

impl<'a> RegisterCurrentHandler<'a> {
    pub fn execute(&self, request: CurrentRequest) {
        self.current_gateway.upsert_current(
            request.name,
            request.version,
            request.vcs_type,
            request.contact,
        );
    }
}

impl<'a> CurrentGateway<'a> {
    pub fn new(settings: &Settings) -> CurrentGateway {
        CurrentGateway { settings }
    }

    pub fn upsert_current(
        &self,
        name: String,
        version: String,
        vcs_type: VcsType,
        contact: String,
    ) {
        let connection = self.settings.database.get_connection();

        // TODO: this should be wrapped somewhere else, because it is going to be used a lot of places. I think there must be some rustic way of doing this
        let vcs_type = match vcs_type {
            VcsType::Github => String::from("github"),
            _ => String::from("Unknown"),
        };

        let mut statement = connection
            .prepare(
                "INSERT INTO Currents (name, version, vcs_type, contact) VALUES(?1,?2,?3,?4) ON CONFLICT(name, vcs_type) DO UPDATE SET version = ?2, contact = ?4",
            )
            .unwrap();

        statement
            .execute([&name, &version, &vcs_type, &contact])
            .unwrap();
    }
}

pub mod models {
    use crate::configuration::VcsType;

    pub struct CurrentRequest {
        pub name: String,
        pub version: String,
        pub vcs_type: VcsType,
        pub contact: String,
    }

    pub struct Current {
        pub name: String,
        pub version: String,
        pub vcs_type: VcsType,
    }
}
